@@ -7,6 +7,7 @@
 """
 import json
 from datetime import datetime
+from functools import partial
 
 import hashlib
 import OpenSSL
@@ -16,6 +17,7 @@ from Cryptodome.Signature import PKCS1_v1_5
 
 from .compat import decodebytes, encodebytes, quote_plus, urlopen
 from .exceptions import AliPayException, AliPayValidationError
+from .utils import AliPayConfig
 
 
 # 常见加密算法
@@ -30,7 +32,7 @@ CryptoAlgSet = (
 )
 
 
-class BaseAliPay(object):
+class BaseAliPay:
     @property
     def appid(self):
         return self._appid
@@ -41,16 +43,12 @@ class BaseAliPay(object):
 
     @property
     def app_private_key(self):
-        """
-        签名用
-        """
+        """签名用"""
         return self._app_private_key
 
     @property
     def alipay_public_key(self):
-        """
-        验证签名用
-        """
+        """验证签名用"""
         return self._alipay_public_key
 
     def __init__(
@@ -60,7 +58,8 @@ class BaseAliPay(object):
         app_private_key_string=None,
         alipay_public_key_string=None,
         sign_type="RSA2",
-        debug=False
+        debug=False,
+        config=None
     ):
         """
         初始化:
@@ -74,14 +73,16 @@ class BaseAliPay(object):
         self._app_notify_url = app_notify_url
         self._app_private_key_string = app_private_key_string
         self._alipay_public_key_string = alipay_public_key_string
+        self._config = config or AliPayConfig()
 
         self._app_private_key = None
         self._alipay_public_key = None
         if sign_type not in ("RSA", "RSA2"):
-            raise AliPayException(None, "Unsupported sign type {}".format(sign_type))
+            message = "Unsupported sign type {}".format(sign_type)
+            raise AliPayException(None, message)
         self._sign_type = sign_type
 
-        if debug is True:
+        if debug:
             self._gateway = "https://openapi.alipaydev.com/gateway.do"
         else:
             self._gateway = "https://openapi.alipay.com/gateway.do"
@@ -103,15 +104,15 @@ class BaseAliPay(object):
         通过如下方法调试签名
         方法1
             key = rsa.PrivateKey.load_pkcs1(open(self._app_private_key_string).read())
-            sign = rsa.sign(unsigned_string.encode("utf8"), key, "SHA-1")
+            sign = rsa.sign(unsigned_string.encode(), key, "SHA-1")
             # base64 编码，转换为unicode表示并移除回车
-            sign = base64.encodebytes(sign).decode("utf8").replace("\n", "")
+            sign = base64.encodebytes(sign).decode().replace("\n", "")
         方法2
             key = RSA.importKey(open(self._app_private_key_string).read())
             signer = PKCS1_v1_5.new(key)
-            signature = signer.sign(SHA.new(unsigned_string.encode("utf8")))
+            signature = signer.sign(SHA.new(unsigned_string.encode()))
             # base64 编码，转换为unicode表示并移除回车
-            sign = base64.encodebytes(signature).decode("utf8").replace("\n", "")
+            sign = base64.encodebytes(signature).decode().replace("\n", "")
         方法3
             echo "abc" | openssl sha1 -sign alipay.key | openssl base64
 
@@ -124,17 +125,15 @@ class BaseAliPay(object):
         else:
             signature = signer.sign(SHA256.new(unsigned_string.encode()))
         # base64 编码，转换为unicode表示并移除回车
-        sign = encodebytes(signature).decode("utf8").replace("\n", "")
+        sign = encodebytes(signature).decode().replace("\n", "")
         return sign
 
     def _ordered_data(self, data):
-        complex_keys = [k for k, v in data.items() if isinstance(v, dict)]
-
-        # 将字典类型的数据dump出来
-        for key in complex_keys:
-            data[key] = json.dumps(data[key], separators=(',', ':'))
-
-        return sorted([(k, v) for k, v in data.items()])
+        for k, v in data.items():
+            if isinstance(v, dict):
+                # 将字典类型的数据dump出来
+                data[k] = json.dumps(v, separators=(',', ':'))
+        return sorted(data.items())
 
     def build_body(
         self, method, biz_content, return_url=None, notify_url=None, append_auth_token=False
@@ -166,12 +165,12 @@ class BaseAliPay(object):
         data.pop("sign", None)
         # 排序后的字符串
         ordered_items = self._ordered_data(data)
-        unsigned_string = "&".join("{}={}".format(k, v) for k, v in ordered_items)
-        sign = self._sign(unsigned_string)
-        quoted_string = "&".join("{}={}".format(k, quote_plus(v)) for k, v in ordered_items)
+        raw_string = "&".join("{}={}".format(k, v) for k, v in ordered_items)
+        sign = self._sign(raw_string)
+        unquoted_items = ordered_items + [('sign', sign)]
 
         # 获得最终的订单信息字符串
-        signed_string = quoted_string + "&sign=" + quote_plus(sign)
+        signed_string = "&".join("{}={}".format(k, quote_plus(v)) for k, v in unquoted_items)
         return signed_string
 
     def _verify(self, raw_content, signature):
@@ -182,10 +181,8 @@ class BaseAliPay(object):
             digest = SHA.new()
         else:
             digest = SHA256.new()
-        digest.update(raw_content.encode("utf8"))
-        if signer.verify(digest, decodebytes(signature.encode("utf8"))):
-            return True
-        return False
+        digest.update(raw_content.encode())
+        return bool(signer.verify(digest, decodebytes(signature.encode())))
 
     def verify(self, data, signature):
         if "sign_type" in data:
@@ -294,10 +291,8 @@ class BaseAliPay(object):
         if trade_no:
             biz_content["trade_no"] = trade_no
         data = self.build_body("alipay.trade.query", biz_content)
-
-        url = self._gateway + "?" + self.sign_data(data)
-        raw_string = urlopen(url, timeout=15).read().decode("utf-8")
-        return self._verify_and_return_sync_response(raw_string, "alipay_trade_query_response")
+        response_type = "alipay_trade_query_response"
+        return self.verified_sync_response(data, response_type)
 
     def api_alipay_trade_pay(
         self, out_trade_no, scene, auth_code, subject, notify_url=None, **kwargs
@@ -362,10 +357,8 @@ class BaseAliPay(object):
         }
         biz_content.update(**kwargs)
         data = self.build_body("alipay.trade.pay", biz_content, notify_url=notify_url)
-
-        url = self._gateway + "?" + self.sign_data(data)
-        raw_string = urlopen(url, timeout=15).read().decode("utf-8")
-        return self._verify_and_return_sync_response(raw_string, "alipay_trade_pay_response")
+        response_type = "alipay_trade_pay_response"
+        return self.verified_sync_response(data, response_type)
 
     def api_alipay_trade_refund(self, refund_amount, out_trade_no=None, trade_no=None, **kwargs):
         biz_content = {
@@ -378,10 +371,8 @@ class BaseAliPay(object):
             biz_content["trade_no"] = trade_no
 
         data = self.build_body("alipay.trade.refund", biz_content)
-
-        url = self._gateway + "?" + self.sign_data(data)
-        raw_string = urlopen(url, timeout=15).read().decode("utf-8")
-        return self._verify_and_return_sync_response(raw_string, "alipay_trade_refund_response")
+        response_type = "alipay_trade_refund_response"
+        return self.verified_sync_response(data, response_type)
 
     def api_alipay_trade_cancel(self, out_trade_no=None, trade_no=None):
         """
@@ -405,10 +396,8 @@ class BaseAliPay(object):
             biz_content["trade_no"] = trade_no
 
         data = self.build_body("alipay.trade.cancel", biz_content)
-
-        url = self._gateway + "?" + self.sign_data(data)
-        raw_string = urlopen(url, timeout=15).read().decode("utf-8")
-        return self._verify_and_return_sync_response(raw_string, "alipay_trade_cancel_response")
+        response_type = "alipay_trade_cancel_response"
+        return self.verified_sync_response(data, response_type)
 
     def api_alipay_trade_close(self, out_trade_no=None, trade_no=None, operator_id=None):
         """
@@ -434,12 +423,25 @@ class BaseAliPay(object):
             biz_content["operator_id"] = operator_id
 
         data = self.build_body("alipay.trade.close", biz_content)
+        response_type = "alipay_trade_close_response"
+        return self.verified_sync_response(data, response_type)
 
-        url = self._gateway + "?" + self.sign_data(data)
-        raw_string = urlopen(url, timeout=15).read().decode("utf-8")
-        return self._verify_and_return_sync_response(raw_string, "alipay_trade_close_response")
+    def api_alipay_trade_create(
+            self, subject, out_trade_no, total_amount, notify_url=None, **kwargs
+    ):
+        biz_content = {
+            "subject": subject,
+            "out_trade_no": out_trade_no,
+            "total_amount": total_amount
+        }
+        biz_content.update(kwargs)
+        data = self.build_body("alipay.trade.create", biz_content)
+        response_type = "alipay_trade_create"
+        return self.verified_sync_response(data, response_type)
 
-    def api_alipay_trade_precreate(self, subject, out_trade_no, total_amount, notify_url=None, **kwargs):
+    def api_alipay_trade_precreate(
+            self, subject, out_trade_no, total_amount, notify_url=None, **kwargs
+    ):
         """
         success response  = {
           "alipay_trade_precreate_response": {
@@ -469,10 +471,8 @@ class BaseAliPay(object):
         }
         biz_content.update(**kwargs)
         data = self.build_body("alipay.trade.precreate", biz_content, notify_url=notify_url)
-
-        url = self._gateway + "?" + self.sign_data(data)
-        raw_string = urlopen(url, timeout=15).read().decode("utf-8")
-        return self._verify_and_return_sync_response(raw_string, "alipay_trade_precreate_response")
+        response_type = "alipay_trade_precreate_response"
+        return self.verified_sync_response(data, response_type)
 
     def api_alipay_trade_fastpay_refund_query(
         self, out_request_no, trade_no=None, out_trade_no=None
@@ -487,12 +487,8 @@ class BaseAliPay(object):
             biz_content["out_trade_no"] = out_trade_no
 
         data = self.build_body("alipay.trade.fastpay.refund.query", biz_content)
-
-        url = self._gateway + "?" + self.sign_data(data)
-        raw_string = urlopen(url, timeout=15).read().decode("utf-8")
-        return self._verify_and_return_sync_response(
-            raw_string, "alipay_trade_fastpay_refund_query_response"
-        )
+        response_type = "alipay_trade_fastpay_refund_query_response"
+        return self.verified_sync_response(data, response_type)
 
     def api_alipay_fund_trans_toaccount_transfer(
             self, out_biz_no, payee_type, payee_account, amount, **kwargs
@@ -506,12 +502,8 @@ class BaseAliPay(object):
         }
         biz_content.update(kwargs)
         data = self.build_body("alipay.fund.trans.toaccount.transfer", biz_content)
-
-        url = self._gateway + "?" + self.sign_data(data)
-        raw_string = urlopen(url, timeout=15).read().decode("utf-8")
-        return self._verify_and_return_sync_response(
-            raw_string, "alipay_fund_trans_toaccount_transfer_response"
-        )
+        response_type = "alipay_fund_trans_toaccount_transfer_response"
+        return self.verified_sync_response(data, response_type)
 
     def api_alipay_fund_trans_order_query(self, out_biz_no=None, order_id=None):
         if out_biz_no is None and order_id is None:
@@ -524,12 +516,8 @@ class BaseAliPay(object):
             biz_content["order_id"] = order_id
 
         data = self.build_body("alipay.fund.trans.order.query", biz_content)
-
-        url = self._gateway + "?" + self.sign_data(data)
-        raw_string = urlopen(url, timeout=15).read().decode("utf-8")
-        return self._verify_and_return_sync_response(
-            raw_string, "alipay_fund_trans_order_query_response"
-        )
+        response_type = "alipay_fund_trans_order_query_response"
+        return self.verified_sync_response(data, response_type)
 
     def api_alipay_trade_order_settle(
         self,
@@ -544,14 +532,9 @@ class BaseAliPay(object):
             "royalty_parameters": royalty_parameters,
         }
         biz_content.update(kwargs)
-
         data = self.build_body("alipay.trade.order.settle", biz_content)
-
-        url = self._gateway + "?" + self.sign_data(data)
-        raw_string = urlopen(url, timeout=15).read().decode("utf-8")
-        return self._verify_and_return_sync_response(
-            raw_string, "alipay_trade_order_settle_response"
-        )
+        response_type = "alipay_trade_order_settle_response"
+        return self.verified_sync_response(data, response_type)
 
     def _verify_and_return_sync_response(self, raw_string, response_type):
         """
@@ -570,7 +553,6 @@ class BaseAliPay(object):
           }
         }
         """
-
         response = json.loads(raw_string)
         # raise exceptions
         if "sign" not in response.keys():
@@ -589,6 +571,11 @@ class BaseAliPay(object):
             raise AliPayValidationError
         return json.loads(plain_content)
 
+    def verified_sync_response(self, data, response_type):
+        url = self._gateway + "?" + self.sign_data(data)
+        raw_string = urlopen(url, timeout=self._config.timeout).read().decode()
+        return self._verify_and_return_sync_response(raw_string, response_type)
+
     def _get_string_to_be_signed(self, raw_string, response_type):
         """
         https://docs.open.alipay.com/200/106120
@@ -617,6 +604,7 @@ class DCAliPay(BaseAliPay):
     """
     数字证书 (digital certificate) 版本
     """
+
     def __init__(
         self,
         appid,
@@ -686,21 +674,15 @@ class DCAliPay(BaseAliPay):
         certIssue = cert.get_issuer()
         name = 'CN={},OU={},O={},C={}'.format(certIssue.CN, certIssue.OU, certIssue.O, certIssue.C)
         string = name + str(cert.get_serial_number())
-        m = hashlib.md5()
-        m.update(bytes(string, encoding="utf8"))
-        return m.hexdigest()
+        return hashlib.md5(string.encode()).hexdigest()
 
     @staticmethod
     def read_pem_cert_chain(certContent):
         """解析根证书"""
-        certs = list()
-        items = certContent.split('\n\n')  # 根证书中，每个 cert 中间有两个回车间隔
-        items = [i for i in items if i]
-
-        for c in items:
-            cert = OpenSSL.crypto.load_certificate(OpenSSL.crypto.FILETYPE_PEM, c)
-            certs.append(cert)
-        return certs
+        # 根证书中，每个 cert 中间有两个回车间隔
+        items = [i for i in certContent.split('\n\n') if i]
+        load_cert = partial(OpenSSL.crypto.load_certificate, OpenSSL.crypto.FILETYPE_PEM)
+        return [load_cert(c) for c in items]
 
     @staticmethod
     def get_root_cert_sn(rootCert):
@@ -718,9 +700,7 @@ class DCAliPay(BaseAliPay):
                     certIssue.CN, certIssue.OU, certIssue.O, certIssue.C
                 )
                 string = name + str(cert.get_serial_number())
-                m = hashlib.md5()
-                m.update(bytes(string, encoding="utf8"))
-                certSN = m.hexdigest()
+                certSN = hashlib.md5(string.encode()).hexdigest()
                 if not rootCertSN:
                     rootCertSN = certSN
                 else:
@@ -758,7 +738,7 @@ class ISVAliPay(BaseAliPay):
 
         self._app_auth_token = app_auth_token
         self._app_auth_code = app_auth_code
-        super(ISVAliPay, self).__init__(
+        super().__init__(
             appid,
             app_notify_url,
             app_private_key_string=app_private_key_string,
@@ -772,19 +752,12 @@ class ISVAliPay(BaseAliPay):
         # 没有则换取token
         if not self._app_auth_token:
             result = self.api_alipay_open_auth_token_app(self._app_auth_code)
-            self._app_auth_token = result.get("app_auth_token", None)
-
-        if not self._app_auth_token:
-            raise Exception("Get auth token by auth code failed: {}".format(self._app_auth_code))
+            self._app_auth_token = result.get("app_auth_token")
+            if not self._app_auth_token:
+                msg = "Get auth token by auth code failed: {}"
+                raise Exception(msg.format(self._app_auth_code))
         return self._app_auth_token
 
-    def build_body(
-        self, method, biz_content, return_url=None, notify_url=None, append_auth_token=True
-    ):
-        return super(ISVAliPay, self).build_body(
-            method, biz_content, return_url, notify_url, append_auth_token
-        )
-
     def api_alipay_open_auth_token_app(self, refresh_token=None):
         """
         response = {
@@ -814,24 +787,15 @@ class ISVAliPay(BaseAliPay):
             biz_content,
             append_auth_token=False
         )
-
-        url = self._gateway + "?" + self.sign_data(data)
-        raw_string = urlopen(url, timeout=15).read().decode("utf-8")
-        return self._verify_and_return_sync_response(
-            raw_string, "alipay_open_auth_token_app_response"
-        )
+        response_type = "alipay_open_auth_token_app_response"
+        return self.verified_sync_response(data, response_type)
 
     def api_alipay_open_auth_token_app_query(self):
-        biz_content = {
-            "app_auth_token": self.app_auth_token
-        }
+        biz_content = {"app_auth_token": self.app_auth_token}
         data = self.build_body(
             "alipay.open.auth.token.app.query",
             biz_content,
             append_auth_token=False
         )
-        url = self._gateway + "?" + self.sign_data(data)
-        raw_string = urlopen(url, timeout=15).read().decode("utf-8")
-        return self._verify_and_return_sync_response(
-            raw_string, "alipay_open_auth_token_app_query_response"
-        )
+        response_type = "alipay_open_auth_token_app_query_response"
+        return self.verified_sync_response(data, response_type)
